﻿'# Title    : SDX9 LenseFlare
'# Author   : Sascha Bajus
'# Created  : 01/2010
'# Copyright © 2010 Sascha Bajus



#Region "Copyright© Information (German)"
' Diese Software wird bereitgestellt "wie sie ist", ohne jegliche ausdrückliche oder 
' stillschweigende Gewährleistung. In keinem Fall kann der Author haftbar gemacht werden
' für Schäden, die aus der Nutzung dieser Software entstanden sind.
'
' Es ist erlaubt, diese Software für jeden Zweck zu nutzen, einschließlich der kommerziellen 
' Anwendungen, und sie zu verändern, vorbehaltlich der folgenden Einschränkungen: 
'
' 1. Der Ursprung dieser Software darf nicht falsch dargestellt werden; Sie dürfen nicht behaupten, 
'    dass Sie der Urheber der Original-Software sind. Wenn Sie diese Software in einem Produkt 

' 2. Altered-Source-Versionen müssen klar als solche gekennzeichnet, und dürfen nicht als 
'    Original-Software dargestellt werden.
'
' 3. Diese Informationen dürfen nicht entfernt oder geändert werden.
#End Region
#Region "Copyright© Information (English)"
' This software is provided 'as-is', without any express or implied warranty. In no event will 
' the authors be held liable for any damages arising from the use of this software.
'
' Permission is granted to anyone to use this software for any purpose, including commercial 
' applications, and to alter it and redistribute it freely, subject to the following restrictions:
'
' 1. The origin of this software must not be misrepresented; you must not claim that you wrote 
'    the original software. If you use this software in a product, an acknowledgment in the 
'    product documentation would be appreciated but is not required.
'
' 2. Altered source versions must be plainly marked as such, and must not be misrepresented 
'    as being the original software.
'
' 3. This notice may not be removed or altered from any source distribution.
'
#End Region

Imports SlimDX
Imports SlimDX.Direct3D9
Imports SlimDX.DirectInput
Imports sdxBaseEngine


Public Class Project : Inherits SDX9BaseEngine

    Private _skydome As New shaderSkydome(24)
    Private _sun As New shaderSun
    Private _floor As New sdxMesh
    Private _floorTex As Texture
    Private _box As Mesh
    Private _boxpositions(4) As Vector3


#Region "New"
    Public Sub New()

        AddHandler dxRenderScene, AddressOf render                      '#Render the Scene
        AddHandler dxKeyboardState, AddressOf keyboard                  '#Get Keyboard Inputs
        AddHandler dxMouseState, AddressOf mouse                        '#Get Mouse Inputs
        AddHandler dxLoadObjectsSettings, AddressOf LoadObjectsSettings '#Load all Settings
        AddHandler dxLoadObjectsTextures, AddressOf LoadObjectsTextures '#Load Textures (Autoreload if Device is Lost)
        AddHandler dxEngineClosed, AddressOf ExitAplication             '#Exit and Dispose all

        '---------------------------------------------

        '#set Cooperate Levels for DirectInput Devices
        'dxEngine.KeyboardSetCooperativeLevel = CooperativeLevel.Exclusive Or CooperativeLevel.Foreground
        dxEngine.MouseSetCooperativeLevel = CooperativeLevel.Exclusive Or CooperativeLevel.Foreground

        '#Enumerate and start Engine
        If dxEngine("SlimDX - SDX9 LenseFlare") = False Then Me.Close()

    End Sub
#End Region
#Region "## Loading ObjectsSettings##"
    Private Sub LoadObjectsSettings()

        dxEngine.ResourceLoader.LoadResourceFile = "data\resx.data"

        '#BaseEngine Skydome with Basic Procedural Clouds
        _skydome.PositionY = -2
        _skydome.CloudSpeed = 0.05

        '#load Xfile for floor
        _floor.MeshFromEngineResource("model\quad.x")
        _floor.MaterialFX(0) = New shaderLight.MaterialSet


        '#Simple DirectX light
        With dxLightsFFP
            .EnableLighting = True
            .EnableDirectionalLight = True
        End With



        '#create box and boxpositions
        _box = Mesh.CreateBox(dxDevice, 1.5, 1.5, 1.5)


        _boxpositions(0) = New Vector3(8, 3, 8)
        _boxpositions(1) = New Vector3(-8, 3, -8)
        _boxpositions(2) = New Vector3(8, 4, -8)
        _boxpositions(3) = New Vector3(-8, 4, 8)
        _boxpositions(4) = New Vector3(0, 4, 0)

        '#sun
        _sun.SunPosition = New Vector3(-50, 30, 0)
        _sun.SunScale = 15
        _sun.OcclusionTestPixel = 1.6


        '#setup camera
        With dxCamera
            Dim _set As engineCamera.cameraSetFirstPerson
            _set.Position = New Vector3(0, 1, 0)
            .useCameraSet(_set)
        End With

    End Sub
#End Region

#Region "## Loading ObjectsTextures ##"
    Private Sub LoadObjectsTextures()
        _floorTex = dxTexture.FromEngineResource("texture\texture.dds", Color.Magenta)
    End Sub
#End Region
#Region "## render ##"
    Private Sub render()
        '#Backbuffer Color
        dxEngine.TargetBufferColor = Color.White

        '#render Skydome
        _skydome.Draw()

        '#render floor
        dxZBuffer.Enabled = True
        dxTransformWorld = Matrix.Translation(New Vector3(0, -1, 0))
        dxDevice.SetTexture(0, _floorTex)
        _floor.Draw(False)

        '#reset floor texture
        dxDevice.SetTexture(0, Nothing)

        '#render rotating boxes
        dxLightsFFP.SetDefaultMaterial()
        For i As Integer = 0 To 4
            dxTransformWorld = dxMatrix.TimebasedScaleTranslateRotate(New Vector3(0, 0.003, 0), _boxpositions(i))
            _box.DrawSubset(0)
        Next
        dxZBuffer.Enabled = False

        'sun
        _sun.DrawSun()
        _sun.DrawFlaires()


        '#render FPS and Text
        dx2DText.DrawFPS(_sun.InfoText, _sun.InfoData)

    End Sub
#End Region

#Region "## Inputdevices ##"
    Private Sub keyboard()

        With dxEngine

            '#Close and Dispose
            If .KeyPressed(Key.Escape) Then CloseBaseEngine()

            '#walk
            If .KeyPressed(Key.LeftControl) Then dxCamera.useKeyboardInput = engineCamera.CamMovements.Forward
            If .KeyPressed(Key.Space) Then dxCamera.useKeyboardInput = engineCamera.CamMovements.Backward

            '#Screenshot
            If .KeyPressed(Key.F12) Then dxEngine.SaveScreenshot("SDX9_LenseFlare")
        End With
    End Sub

    Private Sub mouse()
        '#rotate with mouse
        dxCamera.useMouseRotation = dxEngine.MouseMovement
    End Sub
#End Region
#Region "Exit App"
    Private Sub ExitAplication()
        If Not _box Is Nothing Then _box.Dispose()
        If Not _floorTex Is Nothing Then _floorTex.Dispose()
        If dxEngine.EngineError Then
            Me.Hide()
            MsgBox(dxEngine.EngineErrorDescription, MsgBoxStyle.Critical)
        End If
        End
    End Sub
#End Region

    Private Sub InitializeComponent()
        Dim resources As System.ComponentModel.ComponentResourceManager = New System.ComponentModel.ComponentResourceManager(GetType(Project))
        Me.SuspendLayout()
        '
        'Project
        '
        Me.ClientSize = New System.Drawing.Size(284, 262)
        Me.Icon = CType(resources.GetObject("$this.Icon"), System.Drawing.Icon)
        Me.Name = "Project"
        Me.ResumeLayout(False)

    End Sub
End Class

